/*
 * 	 Constants.java : Commanda line reader
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *	 hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */


import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import org.apache.commons.cli.*;

class OptionSetter
{
	Options options;
	
	public OptionSetter()
	{
		options = new Options();
		init();
	}
	
	private void init()
	{
		Field[] f = Constants.class.getFields();
		for (int i = 0; i < f.length; i++) 
			if (Modifier.isStatic(f[i].getModifiers()))
				options.addOption(OptionBuilder.withLongOpt(f[i].getName()).withValueSeparator(',').hasArgs().create());
	}
}

public class Constants 
{
	//Static variables and option names *must* have the same
	public static boolean help 	 = false;
	
	public static String  ped 	 = null;
	public static String  trt 	 = null;
	public static String  ints 	 = null;
	public static String  mrk    = null;
	
	public static String  output = null;
	public static String  cov 	 = null;
	
	public static String  include = null;
	
	public static boolean verbose= false;		
	public static boolean header = true;   
	public static boolean variance = false;
	
	public static boolean inverse = false;
	public static boolean Ball 	 = false;
	public static int     window = Utilities.INVALID_I;
	public static double  relc 	 = Utilities.INVALID_D;

	public static int permutations = -1;
	public static String  pheno  = null;
	public static int     mpheno = 1;
	
	public static int threads = Integer.MAX_VALUE;
	
	private static Option[] options;
	

	public static void parse(String[] args) throws Exception 
	{
		try 
		{
			Parser parser = new PosixParser();
			OptionSetter opts = new OptionSetter();
			CommandLine cli = parser.parse(opts.options, args);
			options = cli.getOptions();
			
			init(cli);
		}
		catch(MissingArgumentException e) 
		{
			String[] tokens = e.getMessage().split(" ");
			String missingArgument = tokens[tokens.length-1];
			if (missingArgument.equals("help")) //cli require --help true
				help = true;
			else
				throw new Exception("ERROR: Check the " + missingArgument + " parameter: its value might be missing or in the wrong format.\nUse the option --help for details about famCNV usage.");
		}
		catch(ParameterException e)
		{
			throw new Exception(e.getMessage());
		}	
		catch(Exception e)
		{
			throw e;
		}
	}
	
	private static void init(CommandLine cli) 
	{
		if (cli.hasOption("help"))
		{
			help = Boolean.parseBoolean(cli.getOptionValue("help"));
			if (help)
				return;
		}
		
		if (cli.hasOption("ped")) 
			ped = cli.getOptionValue("ped");
		else
			throw new ParameterException("ERROR: ped file has not been specified as input.\nUse the option --help for details about famCNV usage.");

		if (cli.hasOption("trt")) 
			trt = cli.getOptionValue("trt");
		else
			throw new ParameterException("ERROR: trait file has not been specified as input.\nUse the option --help for details about famCNV usage.");

        if (cli.hasOption("ints")) 
			ints = cli.getOptionValue("ints");
		else
			throw new ParameterException("ERROR: intensity file has not been specified as input.\nUse the option --help for details about famCNV usage.");


		if (cli.hasOption("mrk")) 
			mrk = cli.getOptionValue("mrk");
		else
			throw new ParameterException("ERROR: marker file has not been specified as input.\nUse the option --help for details about famCNV usage.");

		if (cli.hasOption("output"))
			output = cli.getOptionValue("output");
			
		if (cli.hasOption("include"))
			include = cli.getOptionValue("include");
		
		if (cli.hasOption("permutations"))
			permutations = Integer.parseInt(cli.getOptionValue("permutations"));

		if (cli.hasOption("Ball"))
			Ball = Boolean.parseBoolean(cli.getOptionValue("Ball"));

		if (cli.hasOption("inverse"))
			inverse = Boolean.parseBoolean(cli.getOptionValue("inverse"));
		
		if (cli.hasOption("verbose"))
		    verbose = Boolean.parseBoolean(cli.getOptionValue("verbose"));
			
		if (cli.hasOption("header"))
			header = Boolean.parseBoolean(cli.getOptionValue("header"));
			
		if (cli.hasOption("variance"))
			variance = Boolean.parseBoolean(cli.getOptionValue("variance"));
			
		if (cli.hasOption("pheno"))
			pheno = cli.getOptionValue("pheno");

		if (cli.hasOption("mpheno")) 
			mpheno = Integer.parseInt(cli.getOptionValue("mpheno")); 		// multiple phenotypes, assigne column
		
		if (cli.hasOption("cov")) 
			cov = cli.getOptionValue("cov");
		
		if (cli.hasOption("window"))
			window = Integer.parseInt(cli.getOptionValue("window"));
		
		if (cli.hasOption("threads"))
			threads = Integer.parseInt(cli.getOptionValue("threads"));
		
		if (cli.hasOption("relc")) 
			relc = Double.parseDouble(cli.getOptionValue("relc"));

	}
	
	public static String optionList()
	{
		String s = "Available options:\n";
		
		s += "\t--help\t\t\tprint this message\n\n";
		s += "\t--ped filepath\t\tped file\n";
		s += "\t--trt filepath\t\ttrait file\n";
		s += "\t--ints filepath\t\tintensity file\n";
		s += "\t--mrk filepath\t\tmarker file\n\n";
		
		s += "\t[--output filepath]\toutput file (default: console)\n";
		s += "\t[--cov filepath]\tcovariate file\n";
		
		s += "\t[--verbose <true|false>]\twhether verbose (default: false)\n";
		s += "\t[--header  <true|false>]\twhether the output file has a header (default: true)\n";
		s += "\t[--variance  <true|false>]\twhether the output shows the variances (default: false)\n";
		
		s += "\t[--Ball <true|false>]\twhether B allele frequencies are available (default: false)\n";
		s += "\t[--inverse <true|false>]\twhether the quantile normalisation is applied to traits(default: false)\n";
		s += "\t[--window bp]\t\twindow overlapping the transcript location (default: none)\n";
		s += "\t[--relc threshold]\twhether creating additional statistics (default: none)\n";
		
		s += "\t[--threads num]\t\tnumber of threads (default: all cores)";
		
		s += "\t[--pheno filepath]\tphenotype file\n";
		s += "\t[--mpheno num]\t\tnumber of phenotypes";
		
		s += "\t[--permutations num]\tnumber of permutaions for evaluating an empiric pvalue (default: 0)\n";
		s += "\t[--include filepath]\tlist of markers for which evaluate an empiric pvalue (defalt: all)\n";
		
		return s;
	}
	
	
	public static String printConstants()
	{
		String s = ""; //"Options in effect:\n";
		
		//print option list 
		for (int i = 0; i < options.length; i++) 
			s += "\t--" + options[i].getLongOpt() + "\t" + options[i].getValue() + "\n";

		return s;
	}
}


class ParameterException extends RuntimeException
{

	public static final long serialVersionUID = 4242L;


	public ParameterException(String msg) 
	{
		super(msg);
	}
}

