/*
 *   Marker.java
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *   hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */

import java.util.*;

 class Marker
 {
    private String chromosome;
    private String name;
    private int position;
    
    public Marker (String name, String chromosome, int position)
    {
	this.chromosome = chromosome;
	this.name = name;
	this.position = position;
    }
    
    public String getName()
    {
	return name;
    }
    
    public String getChromosome()
    {
	return chromosome;
    }
    
    public int getPosition()
    {
	return position;
    }
    
    
    public boolean getAnalysis(Trait T, int window) 
    {
	String Tchr = T.getChromosome();
	int TpositionFrom = T.getPositionFrom();
	int TpositionTo = T.getPositionTo();
	
	boolean wrongPosition = this.position < (TpositionFrom - window) || this.position > (TpositionTo + window);
	boolean wrongChromosome =  Tchr != null && !this.chromosome.equals(Tchr) ;
	
	return (!(window != Utilities.INVALID_I && (wrongPosition || wrongChromosome)));
    }
	
	public boolean isIncluded(Vector<String> list)
	{
		//all markers should be analysed
		if (list.size() == 0)
			return true;
		
		return list.contains(name);
	}
	
}
