/*
 * 	 person.java: Class to keep info for each individual
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *   hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */



import java.util.*;


class Person 
{
	
    private int famId;
    private int id;
    private int idFather;
    private int idMother;
    private int sex;
    private int affection;
    private String twin;
    private int personIndex;
    private int originalIndex;
    private int fatherIndex;
    private int motherIndex;
    private int index;
    private Vector<Double> covariates;
    private Vector<Double> traits;
    private Vector<Double> logR;
    private Vector<Double> bAllele;
    private double phenot;
	private boolean isMock;


    //, Vector<Double> covariates, int personIndex, double F, double variance, int numMarkers
    public Person (int famId, int id, int idFather, int idMother, int sex, int affection, String twin, Vector<Double> traits) 
    {
		this.famId = famId;
		this.id = id;
		this.idFather = idFather;
		this.idMother = idMother;
		this.sex = sex;
		this.affection = affection;
		this.twin = twin;
		this.traits = traits;
	
		personIndex = Utilities.INVALID_I;
		originalIndex = Utilities.INVALID_I;
		phenot = Utilities.INVALID_D;
	
		fatherIndex = 0;
		motherIndex = 0;
		index = 0;
	
		logR = new Vector<Double>();
		bAllele = new Vector<Double>();	
		covariates = new Vector<Double>();
		
		isMock = true;
    }

    public int famId() 
    {
		return famId;
    }
    
    public boolean isMZ()
    {
		return twin.equals("MZ");
    }
	
	public boolean isMock()
	{
		return isMock;
	}
	
	public void setMock(boolean b)
	{
		isMock = b;
	}

    //return whether she has an ancestor (only father is tested)
    public boolean isFounder()
    {
		return idFather == 0;
    }
    
    public int getNumCovariates ()
    {
		return covariates.size();
    }
	
    public int getNumLRRs ()
    {
		return logR.size();
    }
    
    public int originalIndex()
    {
		return originalIndex;
    }
    
    public Vector<Double> covariates()
    {
		return covariates;
    }
    
    public Vector<Double> traits()
    {
		return traits;
    }
	
    public int getNumTraits()
    {
		return traits.size();
    }

    public void setTraits(int i, Double value)
    {
		traits.set(i, value);
    }
    
	public int gender()
	{
		return sex;
	}
	
    public int id()
    {
		return id;
    }
  
    public void setCovariates(Vector<Double> covariates)
    {
		this.covariates=covariates;
    }
    
    public void setPersonIndex(int personIndex)
    {
		this.personIndex = personIndex;
    }
    
    public void setOriginalIndex(int originalIndex)
    {
		this.originalIndex = originalIndex;
    }

    //per riindicizzare il pedigree con i genitori prima dei figli e tutti i founder all'inizio
    public int index()
    {
		return index;
    }
    
    public void setIndex(int index)
    {
		this.index = index;
    }
    
    public int father()
    {
		return idFather;
    }
	
	public void setFather(int i)
	{
		idFather = i;
	}
	
	public void setMother(int i)
	{
		idMother = i;
	}
	
    
    public int fatherIndex()
    {
		return fatherIndex;
    }
    
    public int mother()
    {
		return idMother;
    }
    
    public int motherIndex()
    {
		return motherIndex;
    }
    
   
    public void setFatherIndex(int fatherIndex)
    {
		this.fatherIndex =  fatherIndex;
    }
    
    public void setMotherIndex(int motherIndex)
    {
		this.motherIndex = motherIndex;
    }
    
    public void addLogR(Double v)
    {
	    logR.add(v);
    }
    
    public void addBAllele(Double v)
    {
	    bAllele.add(v);
    }
    
    public double logR(int i)
    {
	    return logR.get(i);
    }
    
    public double bAllele(int i)
    {
	    return bAllele.get(i);
    }
    
    
    public boolean getAnalysis(int trait, int snp)
    {
		if (isMock) 
			return false;
		
		boolean hasTrait = (traits.elementAt(trait)).doubleValue() != Utilities.INVALID_D;
		boolean hasCovariates = !(covariates.size() != 0) || covariates.indexOf(new Double(Utilities.INVALID_D)) == -1;
		boolean hasLogR = logR.get(snp) != Utilities.INVALID_D;
		boolean hasBallele = !Constants.Ball || bAllele.get(snp) != Utilities.INVALID_D;
	
		return hasTrait && hasCovariates && hasLogR && hasBallele;
    }
    
    //Is this used somewhere? Family has a fixme regarding this value
    public int affection()
    {
		return affection;
    }

}
