/*
 * 	 Result.java
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 * 	 visconti@di.unito.it
 */

import java.text.*;

class Result 
{
		
	private static final String header = "Trait\tChr\tFrom\tTo\tMarker\tPosition\tLnlk_Null\tLnlk_Full\tdf_Null\tdf_Full\tchi^2\tP\tbeta\tvariance explained";
	
	private static final DecimalFormat df2 = new DecimalFormat("####0.00");
	private static final DecimalFormat df4 = new DecimalFormat("0.0000");
	private static final DecimalFormat df3 = new DecimalFormat("0.000");
	private static final DecimalFormat dfe = new DecimalFormat("0.00E0");
	private static final DecimalFormat dfp = new DecimalFormat("##0.00%");
			
	
	private String trait;
	private int trtFrom;
	private int trtTo;
	
	private String marker;
	private int mrkPosition;
	
	private String chr;

	private double lnlkNull;
	private double lnlkFull;
	private int dfNull;
	private int dfFull;
	private double chiq;
	private double pvalue;
	private double epvalue;
	private double posF;
	private double giniC;
	private double beta;
	private double ve;
	private double[] varNull;
	private double[] varFull;
	private String emessage;
	
	
	public Result(double sNull, double sFull, int dfN, int dfF, double test, double pval, double epv, double pF, double gini, double b, double v, double[] vNull, double[] vFull)
	{
		trait = null;
		marker = null;
		chr = null;
		mrkPosition = -1;
		trtFrom = -1;
		trtTo = -1;
		
		lnlkNull = sNull;
		lnlkFull = sFull;
		dfNull = dfN;
		dfFull = dfF;
		chiq = test;
		pvalue = pval;
		epvalue = epv;
		posF = pF;
		giniC = gini;
		beta = b;
		ve = v;
		
		varNull = vNull;
		varFull =vFull;
		
		emessage = null;
	}
	
	
	public Result(String emsg)
	{
		emessage = emsg;
	}
	
	
	public void setInfo(String t, String m, String c, int p, int from, int to)
	{
		trait = t;
		marker = m;
		chr = c;
		mrkPosition = p;
		trtFrom = from;
		trtTo = to;
	}
	
	
	public static String header(boolean epv, boolean rec, boolean variance)
	{
		String r = header;
		
		if (epv)
			r += "\tePvalue";
		
		if (rec)
			r += "\t%PosF\tGiniC";
		
		if (variance)
			r += "\tvar_Null\tvar_Full";
			
		return r;
	}
	
	public String to_string() throws Exception //toString ->  overridden method does not throw Exception
	{
		if (trait == null)
			throw new Exception("Internal error: not well formed result");
		
		String result = trait + "\t" + chr + "\t" + trtFrom + "\t" + trtTo + "\t" + marker + "\t" + mrkPosition + "\t";
		
		if (emessage != null) //exception thrown
		{
			result += emessage;
			return result; 
		}
			
		result += df2.format(lnlkNull) + "\t" + df2.format(lnlkFull) + "\t" + dfNull + "\t" + dfFull + "\t";
		result += df2.format(chiq) + "\t" + (pvalue < 0.0001 ? dfe.format(pvalue) : df4.format(pvalue)) + "\t";
		result += df3.format(beta) + "\t" + dfp.format(ve);

		if (epvalue != Utilities.INVALID_D)
			result += "\t" + (epvalue < 0.0001 ? dfe.format(epvalue) : df4.format(epvalue));

		if (posF != Utilities.INVALID_D)
			result += "\t" + dfp.format(posF) + "\t" + df2.format(giniC);

		if (Constants.variance)
		{
			String vNull = Utilities.printArray(varNull, df3);
			String vFull = Utilities.printArray(varFull, df3);
		
			result += "\t" + vNull +  "\t" + vFull;
		}

		return result;
	}
	

	
		
}
	
